from pydantic import BaseModel
from typing import List, Optional
from datetime import datetime


class OrderItemCreate(BaseModel):
    product_id: int
    quantity: int


class CartItemCreate(BaseModel):
    shop_id: int
    product_id: int
    quantity: int
    price: Optional[float] = None


class CartItemRead(BaseModel):
    id: int
    product_id: int
    quantity: int
    price: float

    class Config:
        from_attributes = True


class CartRead(BaseModel):
    id: int
    user_id: int
    created_at: datetime
    cart_items: List[CartItemRead]

    class Config:
        from_attributes = True


class OrderCreate(BaseModel):
    shop_id: int
    payment_id: int
    items: List[OrderItemCreate]
    delivery_address: str


class OrderItemRead(BaseModel):
    id: int
    product_id: int
    quantity: int
    price: float

    class Config:
        from_attributes = True


class OrderRead(BaseModel):
    id: int
    user_id: int
    shop_id: int
    total_price: float
    shipping_price: float
    status: str
    created_at: datetime
    order_items: List[OrderItemRead]

    class Config:
        from_attributes = True


class OrderUpdate(BaseModel):
    delivery_address: Optional[str] = None
    status: Optional[str] = None
