import customtkinter as ctk
import requests
from tkinter import messagebox
from PIL import Image, ImageTk
import io

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"
SIDEBAR_BG = "#2D3035"


def owner_dashboard_frame(parent, switch_func, API_URL, token):
    """
    Shop owner dashboard with revenue tracking and product management.
    """
    # Main container frame
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)

    # Track auto-refresh jobs for cleanup
    refresh_job_ids = []

    def update_token(new_token):
        nonlocal token
        token = new_token

    frame.update_token = update_token  # Add method to the frame

    # Create layout: sidebar + main content
    sidebar_frame = ctk.CTkFrame(frame, fg_color=SIDEBAR_BG, width=180)
    sidebar_frame.pack(side="left", fill="y")
    sidebar_frame.pack_propagate(False)  # Don't shrink

    main_frame = ctk.CTkFrame(frame, fg_color=DARK_BG)
    main_frame.pack(side="left", fill="both", expand=True)

    # --- SIDEBAR CONTENT ---
    sidebar_title = ctk.CTkLabel(
        sidebar_frame,
        text="Shop\nDashboard",
        font=("Helvetica", 18, "bold"),
        text_color="white",
    )
    sidebar_title.pack(pady=(20, 30))

    def create_sidebar_button(text, command=None, is_active=False):
        return ctk.CTkButton(
            sidebar_frame,
            text=text,
            command=command,
            fg_color=SHOPPING if is_active else "transparent",
            text_color="white",
            hover_color="#3b3b3b" if not is_active else None,
            height=40,
            anchor="w",
            font=("Helvetica", 14),
        )

    # Active button for revenue dashboard
    revenue_btn = create_sidebar_button("Revenue", is_active=True)
    revenue_btn.pack(fill="x", pady=5, padx=10)

    # Button for product management
    def open_product_management():
        switch_func("owner_products")

    product_btn = create_sidebar_button("Product Management", open_product_management)
    product_btn.pack(fill="x", pady=5, padx=10)

    # Button for order management
    def open_order_management():
        switch_func("owner_orders")

    orders_btn = create_sidebar_button("Order Management", open_order_management)
    orders_btn.pack(fill="x", pady=5, padx=10)

    # Button to go to buyer dashboard
    def go_to_shop():
        # Use the dashboard_bypass flag to avoid role-based routing
        switch_func("dashboard_bypass", token)

    shop_btn = create_sidebar_button("Go to Shop", go_to_shop)
    shop_btn.pack(fill="x", pady=5, padx=10)

    # Logout button at bottom of sidebar
    def logout():
        switch_func("login")

    logout_btn = ctk.CTkButton(
        sidebar_frame,
        text="Logout",
        command=logout,
        fg_color="#555",
        text_color="white",
        hover_color="#777",
        height=40,
    )
    logout_btn.pack(side="bottom", fill="x", pady=20, padx=10)

    # --- MAIN CONTENT ---

    # Shop Information Section
    shop_info_frame = ctk.CTkFrame(main_frame, fg_color=CARD_BG, corner_radius=15)
    shop_info_frame.pack(fill="x", padx=30, pady=20)

    # Shop Info Header
    shop_info_header = ctk.CTkFrame(shop_info_frame, fg_color="transparent")
    shop_info_header.pack(fill="x", padx=20, pady=15)

    shop_info_title = ctk.CTkLabel(
        shop_info_header,
        text="Shop Information",
        font=("Helvetica", 20, "bold"),
        text_color=SHOPPING,
    )
    shop_info_title.pack(side="left")

    # Shop Info Content
    shop_info_content = ctk.CTkFrame(shop_info_frame, fg_color="transparent")
    shop_info_content.pack(fill="x", padx=20, pady=(0, 20))

    # Left side: Shop Logo
    logo_frame = ctk.CTkFrame(
        shop_info_content, fg_color="transparent", width=180, height=180
    )
    logo_frame.pack(side="left", padx=(0, 20))
    logo_frame.pack_propagate(False)

    logo_placeholder = ctk.CTkLabel(
        logo_frame,
        text="Shop Logo",
        fg_color="#3b3b3b",
        corner_radius=10,
        font=("Helvetica", 14),
        text_color="gray",
    )
    logo_placeholder.pack(fill="both", expand=True, padx=5, pady=5)

    # Right side: Shop Details
    details_frame = ctk.CTkFrame(shop_info_content, fg_color="transparent")
    details_frame.pack(side="left", fill="both", expand=True)

    # Shop Name
    name_label = ctk.CTkLabel(
        details_frame,
        text="Shop Name:",
        font=("Helvetica", 14, "bold"),
        text_color="white",
    )
    name_label.grid(row=0, column=0, sticky="w", pady=(0, 10))

    shop_name_value = ctk.CTkLabel(
        details_frame, text="Loading...", font=("Helvetica", 14), text_color="white"
    )
    shop_name_value.grid(row=0, column=1, sticky="w", padx=(10, 0), pady=(0, 10))

    # Shop Description
    desc_label = ctk.CTkLabel(
        details_frame,
        text="Description:",
        font=("Helvetica", 14, "bold"),
        text_color="white",
    )
    desc_label.grid(row=1, column=0, sticky="nw", pady=(0, 10))

    shop_desc_value = ctk.CTkLabel(
        details_frame,
        text="Loading...",
        font=("Helvetica", 14),
        text_color="white",
        wraplength=400,
        justify="left",
    )
    shop_desc_value.grid(row=1, column=1, sticky="w", padx=(10, 0), pady=(0, 10))

    # Shop Address
    addr_label = ctk.CTkLabel(
        details_frame,
        text="Address:",
        font=("Helvetica", 14, "bold"),
        text_color="white",
    )
    addr_label.grid(row=2, column=0, sticky="nw", pady=(0, 10))

    shop_addr_value = ctk.CTkLabel(
        details_frame,
        text="Loading...",
        font=("Helvetica", 14),
        text_color="white",
        wraplength=400,
        justify="left",
    )
    shop_addr_value.grid(row=2, column=1, sticky="w", padx=(10, 0), pady=(0, 10))

    details_frame.grid_columnconfigure(1, weight=1)

    # Revenue Analysis Title
    title_label = ctk.CTkLabel(
        main_frame,
        text="Revenue Analysis",
        font=("Helvetica", 24, "bold"),
        text_color="white",
    )
    title_label.pack(anchor="w", padx=30, pady=(20, 20))

    # Revenue stats cards
    stats_frame = ctk.CTkFrame(main_frame, fg_color="transparent")
    stats_frame.pack(fill="x", padx=30, pady=10)

    # Order count card
    order_card = ctk.CTkFrame(stats_frame, fg_color="white", corner_radius=10)
    order_card.pack(side="left", fill="both", expand=True, padx=(0, 15))

    order_title = ctk.CTkLabel(
        order_card,
        text="All-time Shipped Orders",
        font=("Helvetica", 16),
        text_color="#333",
    )
    order_title.pack(anchor="w", padx=20, pady=(20, 5))

    order_subtitle = ctk.CTkLabel(
        order_card,
        text="Total orders ever shipped, delivered, or completed",
        font=("Helvetica", 10),
        text_color="#666",
    )
    order_subtitle.pack(anchor="w", padx=20, pady=(0, 5))

    order_count = ctk.CTkLabel(
        order_card, text="0", font=("Helvetica", 36, "bold"), text_color="#333"
    )
    order_count.pack(anchor="w", padx=20, pady=(0, 20))

    # Revenue card
    revenue_card = ctk.CTkFrame(stats_frame, fg_color="white", corner_radius=10)
    revenue_card.pack(side="left", fill="both", expand=True, padx=(15, 0))

    revenue_title = ctk.CTkLabel(
        revenue_card,
        text="All-time Product Revenue ($)",
        font=("Helvetica", 16),
        text_color="#333",
    )
    revenue_title.pack(anchor="w", padx=20, pady=(20, 5))

    revenue_subtitle = ctk.CTkLabel(
        revenue_card,
        text="Total product revenue (excluding shipping fees)",
        font=("Helvetica", 10),
        text_color="#666",
    )
    revenue_subtitle.pack(anchor="w", padx=20, pady=(0, 5))

    revenue_amount = ctk.CTkLabel(
        revenue_card, text="0.00", font=("Helvetica", 36, "bold"), text_color="#333"
    )
    revenue_amount.pack(anchor="w", padx=20, pady=(0, 20))

    # Product Management Section
    products_section = ctk.CTkFrame(main_frame, fg_color="transparent")
    products_section.pack(fill="both", expand=True, padx=30, pady=20)

    # Header with title
    products_header = ctk.CTkFrame(products_section, fg_color="transparent")
    products_header.pack(fill="x", pady=10)

    products_title = ctk.CTkLabel(
        products_header,
        text="Recent Products",
        font=("Helvetica", 20, "bold"),
        text_color="white",
    )
    products_title.pack(side="left")

    # Products table
    products_table_container = ctk.CTkFrame(products_section, fg_color="transparent")
    products_table_container.pack(fill="both", expand=True, pady=10)

    # Make the products table scrollable
    products_scrollable = ctk.CTkScrollableFrame(
        products_table_container, fg_color=SIDEBAR_BG, corner_radius=5
    )
    products_scrollable.pack(fill="both", expand=True)

    products_table = ctk.CTkFrame(products_scrollable, fg_color=SIDEBAR_BG)
    products_table.pack(fill="both", expand=True)

    # Add mouse wheel scroll binding for better scrolling
    def _bound_to_mousewheel(event):
        products_scrollable._parent_canvas.bind_all("<MouseWheel>", _on_mousewheel)

    def _unbound_to_mousewheel(event):
        products_scrollable._parent_canvas.unbind_all("<MouseWheel>")

    def _on_mousewheel(event):
        products_scrollable._parent_canvas.yview_scroll(
            int(-1 * (event.delta / 120)), "units"
        )

    # Bind these functions to the frame
    products_scrollable.bind("<Enter>", _bound_to_mousewheel)
    products_scrollable.bind("<Leave>", _unbound_to_mousewheel)

    # Table headers
    headers_frame = ctk.CTkFrame(products_table, fg_color="#1a1a1a", height=40)
    headers_frame.pack(fill="x")

    # Create the headers
    header_texts = ["Image", "Title", "Product Name", "Price", "Quantity"]
    header_widths = [0.15, 0.25, 0.25, 0.15, 0.2]  # Adjusted proportional widths

    for i, text in enumerate(header_texts):
        header_cell = ctk.CTkFrame(headers_frame, fg_color="transparent")
        header_cell.place(
            relx=sum(header_widths[:i]), rely=0, relwidth=header_widths[i], relheight=1
        )

        header_label = ctk.CTkLabel(
            header_cell, text=text, font=("Helvetica", 14, "bold"), text_color="white"
        )
        header_label.place(relx=0.5, rely=0.5, anchor="center")

    # Container for product rows
    product_rows_frame = ctk.CTkFrame(products_table, fg_color="transparent")
    product_rows_frame.pack(fill="both", expand=True)

    # Product rows list to track and update
    product_rows = []

    def create_product_row(product):
        """Create a row for a product in the table"""
        row = ctk.CTkFrame(product_rows_frame, fg_color="transparent", height=60)
        row.pack(fill="x", pady=1)
        row.pack_propagate(False)

        # Image cell
        img_cell = ctk.CTkFrame(row, fg_color="transparent")
        img_cell.place(relx=0, rely=0, relwidth=header_widths[0], relheight=1)

        img_label = ctk.CTkLabel(img_cell, text="No Image")
        img_label.place(relx=0.5, rely=0.5, anchor="center")

        # Try to load the image if available
        if product.get("images") and len(product["images"]) > 0:
            image_url = product["images"][0].get("image_url")
            if image_url:
                try:
                    resp = requests.get(image_url)
                    if resp.status_code == 200:
                        pil_img = Image.open(io.BytesIO(resp.content)).resize((40, 40))
                        tk_img = ImageTk.PhotoImage(pil_img)
                        img_label.configure(image=tk_img, text="")
                        img_label.image = tk_img
                except Exception as e:
                    print(f"[DEBUG] Product image error: {e}")

        # Title cell (description)
        title_cell = ctk.CTkFrame(row, fg_color="transparent")
        title_cell.place(
            relx=header_widths[0], rely=0, relwidth=header_widths[1], relheight=1
        )

        title_label = ctk.CTkLabel(
            title_cell,
            text=product.get("description", "No description")[:50] + "..."
            if product.get("description", "")
            and len(product.get("description", "")) > 50
            else product.get("description", "No description"),
            font=("Helvetica", 12),
            text_color="white",
            wraplength=180,
        )
        title_label.place(relx=0.5, rely=0.5, anchor="center")

        # Product name cell
        name_cell = ctk.CTkFrame(row, fg_color="transparent")
        name_cell.place(
            relx=sum(header_widths[:2]), rely=0, relwidth=header_widths[2], relheight=1
        )

        name_label = ctk.CTkLabel(
            name_cell,
            text=product.get("name", "No name"),
            font=("Helvetica", 12),
            text_color="white",
        )
        name_label.place(relx=0.5, rely=0.5, anchor="center")

        # Price cell
        price_cell = ctk.CTkFrame(row, fg_color="transparent")
        price_cell.place(
            relx=sum(header_widths[:3]), rely=0, relwidth=header_widths[3], relheight=1
        )

        price_label = ctk.CTkLabel(
            price_cell,
            text=f"${product.get('price', 0):.2f}",
            font=("Helvetica", 12),
            text_color="#ff4242",  # Red color for price
        )
        price_label.place(relx=0.5, rely=0.5, anchor="center")

        # Quantity cell
        qty_cell = ctk.CTkFrame(row, fg_color="transparent")
        qty_cell.place(
            relx=sum(header_widths[:4]), rely=0, relwidth=header_widths[4], relheight=1
        )

        qty_label = ctk.CTkLabel(
            qty_cell,
            text=str(product.get("stock", 0)),
            font=("Helvetica", 12),
            text_color="white",
        )
        qty_label.place(relx=0.5, rely=0.5, anchor="center")

        return row

    def clear_product_rows():
        """Clear all product rows from the table"""
        for row in product_rows:
            row.destroy()
        product_rows.clear()

    # Now fetch shop statistics
    def fetch_shop_statistics(shop_id):
        """Fetch only the shop statistics to update the dashboard metrics"""
        headers = {"Authorization": f"Bearer {token}"}
        try:
            stats_resp = requests.get(
                f"{API_URL}/shops/stats/{shop_id}", headers=headers
            )
            if stats_resp.status_code == 200:
                stats = stats_resp.json()
                # Update UI with real data
                shipped_orders = stats.get("shipped_orders", 0)
                order_count.configure(text=str(shipped_orders))
                # Convert to display dollars with 2 decimal places
                total_revenue = stats.get("total_revenue", 0)
                revenue_amount.configure(text=f"{total_revenue:,.2f}")

                # Schedule the next auto-refresh
                job_id = parent.after(30000, lambda: fetch_shop_statistics(shop_id))
                refresh_job_ids.append(job_id)
        except Exception as e:
            print(f"Error refreshing shop statistics: {e}")
            # Even on error, try to refresh again later
            job_id = parent.after(60000, lambda: fetch_shop_statistics(shop_id))
            refresh_job_ids.append(job_id)

    # Fetch shop details and products
    def fetch_shop_data():
        headers = {"Authorization": f"Bearer {token}"}
        try:
            # Get owner's shop
            resp = requests.get(f"{API_URL}/shops/owner", headers=headers)
            if resp.status_code == 200:
                shops_data = resp.json()
                if shops_data:
                    # Use the first shop
                    shop = shops_data[0]

                    # Update shop info section
                    shop_name_value.configure(text=shop.get("name", "No name"))
                    shop_desc_value.configure(
                        text=shop.get("description", "No description")
                    )
                    shop_addr_value.configure(text=shop.get("address", "No address"))

                    # Load shop logo
                    shop_image_url = shop.get("image_url")
                    if shop_image_url:
                        try:
                            resp = requests.get(shop_image_url)
                            if resp.status_code == 200:
                                pil_img = Image.open(io.BytesIO(resp.content)).resize(
                                    (170, 170)
                                )
                                tk_img = ImageTk.PhotoImage(pil_img)
                                logo_placeholder.configure(image=tk_img, text="")
                                logo_placeholder.image = tk_img
                        except Exception as e:
                            print(f"Error loading shop logo: {e}")

                    # Now fetch shop statistics
                    stats_resp = requests.get(
                        f"{API_URL}/shops/stats/{shop['id']}", headers=headers
                    )
                    if stats_resp.status_code == 200:
                        stats = stats_resp.json()
                        # Update UI with real data
                        shipped_orders = stats.get("shipped_orders", 0)
                        order_count.configure(text=str(shipped_orders))
                        # Convert to display dollars with 2 decimal places
                        total_revenue = stats.get("total_revenue", 0)
                        revenue_amount.configure(text=f"{total_revenue:,.2f}")

                        # Start auto-refresh for statistics
                        job_id = parent.after(
                            30000, lambda: fetch_shop_statistics(shop["id"])
                        )
                        refresh_job_ids.append(job_id)

                    # Fetch shop products
                    products_resp = requests.get(
                        f"{API_URL}/product/shop/{shop['id']}", headers=headers
                    )
                    if products_resp.status_code == 200:
                        products = products_resp.json()

                        # Clear existing product rows
                        clear_product_rows()

                        # Create new rows for recent products (up to 5)
                        recent_products = products[:5] if products else []

                        if recent_products:
                            for product in recent_products:
                                row = create_product_row(product)
                                product_rows.append(row)
                        else:
                            # Display a message if no products
                            no_products_label = ctk.CTkLabel(
                                product_rows_frame,
                                text="No products found. Add products in the Product Management section.",
                                font=("Helvetica", 14),
                                text_color="gray",
                            )
                            no_products_label.pack(pady=20)
                            product_rows.append(no_products_label)
                else:
                    # No shop found
                    shop_name_value.configure(text="You don't have a shop")
                    shop_desc_value.configure(
                        text="Please create a shop to start selling"
                    )
                    shop_addr_value.configure(text="")

                    # Clear any product rows and show a message
                    clear_product_rows()
                    no_shop_label = ctk.CTkLabel(
                        product_rows_frame,
                        text="Create a shop to start adding products",
                        font=("Helvetica", 14),
                        text_color="gray",
                    )
                    no_shop_label.pack(pady=20)
                    product_rows.append(no_shop_label)
        except Exception as e:
            print(f"Error fetching shop data: {e}")
            messagebox.showerror("Error", f"Failed to load shop data: {e}")

    # Initialize by loading data
    fetch_shop_data()  # Load shop data on startup

    # Add refresh method to frame for external calls
    def refresh_data(*args):
        # Clear existing content and fetch fresh data
        try:
            print("Refreshing owner dashboard...")
            # Cancel any existing auto-refresh jobs
            for job_id in refresh_job_ids:
                parent.after_cancel(job_id)
            refresh_job_ids.clear()

            clear_product_rows()
            fetch_shop_data()  # This will restart the auto-refresh cycle
            print("Owner dashboard refreshed with latest data")
        except Exception as e:
            print(f"Error refreshing dashboard: {e}")

    frame.refresh_data = refresh_data

    # Add cleanup method to frame
    def cleanup():
        for job_id in refresh_job_ids:
            parent.after_cancel(job_id)
        refresh_job_ids.clear()

    frame.cleanup = cleanup

    return frame
