import customtkinter as ctk
import requests
from tkinter import messagebox
from PIL import Image, ImageTk
import io

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"
SIDEBAR_BG = "#2D3035"


def owner_products_frame(parent, switch_func, API_URL, token):
    """
    Product management page for shop owners.
    """
    # Main container frame
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)

    def update_token(new_token):
        nonlocal token
        token = new_token

    frame.update_token = update_token  # Add method to the frame

    # Create layout: sidebar + main content
    sidebar_frame = ctk.CTkFrame(frame, fg_color=SIDEBAR_BG, width=180)
    sidebar_frame.pack(side="left", fill="y")
    sidebar_frame.pack_propagate(False)  # Don't shrink

    main_frame = ctk.CTkFrame(frame, fg_color=DARK_BG)
    main_frame.pack(side="left", fill="both", expand=True)

    # --- SIDEBAR CONTENT ---
    sidebar_title = ctk.CTkLabel(
        sidebar_frame,
        text="Shop\nDashboard",
        font=("Helvetica", 18, "bold"),
        text_color="white",
    )
    sidebar_title.pack(pady=(20, 30))

    def create_sidebar_button(text, command=None, is_active=False):
        return ctk.CTkButton(
            sidebar_frame,
            text=text,
            command=command,
            fg_color=SHOPPING if is_active else "transparent",
            text_color="white",
            hover_color="#3b3b3b" if not is_active else None,
            height=40,
            anchor="w",
            font=("Helvetica", 14),
        )

    # Button for revenue dashboard
    def open_revenue_dashboard():
        switch_func("owner_dashboard")

    revenue_btn = create_sidebar_button("Revenue", open_revenue_dashboard)
    revenue_btn.pack(fill="x", pady=5, padx=10)

    # Active button for product management
    product_btn = create_sidebar_button("Product Management", is_active=True)
    product_btn.pack(fill="x", pady=5, padx=10)

    # Button for order management
    def open_order_management():
        switch_func("owner_orders")

    orders_btn = create_sidebar_button("Order Management", open_order_management)
    orders_btn.pack(fill="x", pady=5, padx=10)

    # Button to go to buyer dashboard
    def go_to_shop():
        # Use the dashboard_bypass flag to avoid role-based routing
        switch_func("dashboard_bypass", token)

    shop_btn = create_sidebar_button("Go to Shop", go_to_shop)
    shop_btn.pack(fill="x", pady=5, padx=10)

    # Logout button at bottom of sidebar
    def logout():
        switch_func("login")

    logout_btn = ctk.CTkButton(
        sidebar_frame,
        text="Logout",
        command=logout,
        fg_color="#555",
        text_color="white",
        hover_color="#777",
        height=40,
    )
    logout_btn.pack(side="bottom", fill="x", pady=20, padx=10)

    # --- MAIN CONTENT ---
    # Title
    title_label = ctk.CTkLabel(
        main_frame,
        text="Product Management",
        font=("Helvetica", 24, "bold"),
        text_color="white",
    )
    title_label.pack(anchor="w", padx=30, pady=(30, 20))

    # Add Product button
    button_frame = ctk.CTkFrame(main_frame, fg_color="transparent")
    button_frame.pack(fill="x", padx=30, pady=(0, 20))

    def add_new_product():
        switch_func("create_product")

    add_button = ctk.CTkButton(
        button_frame,
        text="Add Product",
        command=add_new_product,
        fg_color=SHOPPING,
        hover_color="#0096ff",
        height=40,
        width=150,
    )
    add_button.pack(side="right")

    # Products table
    products_frame = ctk.CTkScrollableFrame(main_frame, fg_color="transparent")
    products_frame.pack(fill="both", expand=True, padx=30, pady=(0, 20))

    # Table container
    table_frame = ctk.CTkFrame(products_frame, fg_color=SIDEBAR_BG, corner_radius=5)
    table_frame.pack(fill="both", expand=True)

    # Fix the scrolling function - use the _bound_to_mousewheel approach instead
    def _bound_to_mousewheel(event):
        products_frame._parent_canvas.bind_all("<MouseWheel>", _on_mousewheel)

    def _unbound_to_mousewheel(event):
        products_frame._parent_canvas.unbind_all("<MouseWheel>")

    def _on_mousewheel(event):
        products_frame._parent_canvas.yview_scroll(
            int(-1 * (event.delta / 120)), "units"
        )

    # Bind these functions to the frame
    products_frame.bind("<Enter>", _bound_to_mousewheel)
    products_frame.bind("<Leave>", _unbound_to_mousewheel)

    # Table headers
    headers_frame = ctk.CTkFrame(table_frame, fg_color="#1a1a1a", height=40)
    headers_frame.pack(fill="x")
    headers_frame.pack_propagate(False)

    # Create the headers
    header_texts = ["Image", "Title", "Product Name", "Price", "Quantity", "Action"]
    header_widths = [
        0.15,
        0.2,
        0.2,
        0.15,
        0.1,
        0.2,
    ]  # Proportional widths adjusted for price

    for i, text in enumerate(header_texts):
        header_cell = ctk.CTkFrame(headers_frame, fg_color="transparent")
        header_cell.place(
            relx=sum(header_widths[:i]), rely=0, relwidth=header_widths[i], relheight=1
        )

        header_label = ctk.CTkLabel(
            header_cell, text=text, font=("Helvetica", 14, "bold"), text_color="white"
        )
        header_label.place(relx=0.5, rely=0.5, anchor="center")

    # Store shop data and product rows
    shop_data = None
    product_rows = []

    def create_product_row(product):
        """Create a row for a product in the table"""
        row = ctk.CTkFrame(table_frame, fg_color="transparent", height=70)
        row.pack(fill="x", pady=1)
        row.pack_propagate(False)

        # Image cell
        img_cell = ctk.CTkFrame(row, fg_color="transparent")
        img_cell.place(relx=0, rely=0, relwidth=header_widths[0], relheight=1)

        img_label = ctk.CTkLabel(img_cell, text="No Image")
        img_label.place(relx=0.5, rely=0.5, anchor="center")

        # Try to load the image if available
        if product.get("images") and len(product["images"]) > 0:
            image_url = product["images"][0].get("image_url")
            if image_url:
                try:
                    resp = requests.get(image_url)
                    if resp.status_code == 200:
                        pil_img = Image.open(io.BytesIO(resp.content)).resize((50, 50))
                        tk_img = ImageTk.PhotoImage(pil_img)
                        img_label.configure(image=tk_img, text="")
                        img_label.image = tk_img
                except Exception as e:
                    print(f"Error loading product image: {e}")
                    img_label.configure(text="No Image")
        else:
            img_label.configure(text="No Image")

        # Title cell
        title_cell = ctk.CTkFrame(row, fg_color="transparent")
        title_cell.place(
            relx=header_widths[0], rely=0, relwidth=header_widths[1], relheight=1
        )

        title_label = ctk.CTkLabel(
            title_cell,
            text=product.get("description", "No description")[:50] + "..."
            if product.get("description", "")
            and len(product.get("description", "")) > 50
            else product.get("description", "No description"),
            font=("Helvetica", 12),
            text_color="white",
            wraplength=150,
        )
        title_label.place(relx=0.5, rely=0.5, anchor="center")

        # Product name cell
        name_cell = ctk.CTkFrame(row, fg_color="transparent")
        name_cell.place(
            relx=sum(header_widths[:2]), rely=0, relwidth=header_widths[2], relheight=1
        )

        name_label = ctk.CTkLabel(
            name_cell,
            text=product.get("name", "No name"),
            font=("Helvetica", 12),
            text_color="white",
        )
        name_label.place(relx=0.5, rely=0.5, anchor="center")

        # Price cell
        price_cell = ctk.CTkFrame(row, fg_color="transparent")
        price_cell.place(
            relx=sum(header_widths[:3]), rely=0, relwidth=header_widths[3], relheight=1
        )

        price_label = ctk.CTkLabel(
            price_cell,
            text=f"${product.get('price', 0):.2f}",
            font=("Helvetica", 12),
            text_color="#ff4242",  # Red color for price
        )
        price_label.place(relx=0.5, rely=0.5, anchor="center")

        # Quantity cell
        qty_cell = ctk.CTkFrame(row, fg_color="transparent")
        qty_cell.place(
            relx=sum(header_widths[:4]), rely=0, relwidth=header_widths[4], relheight=1
        )

        qty_label = ctk.CTkLabel(
            qty_cell,
            text=str(product.get("stock", 0)),
            font=("Helvetica", 12),
            text_color="white",
        )
        qty_label.place(relx=0.5, rely=0.5, anchor="center")

        # Action cell
        action_cell = ctk.CTkFrame(row, fg_color="transparent")
        action_cell.place(
            relx=sum(header_widths[:5]), rely=0, relwidth=header_widths[5], relheight=1
        )

        # Action buttons container (centered within action cell)
        buttons_frame = ctk.CTkFrame(action_cell, fg_color="transparent")
        buttons_frame.place(relx=0.5, rely=0.5, anchor="center")

        def edit_product():
            switch_func("edit_product", product["id"])

        edit_btn = ctk.CTkButton(
            buttons_frame,
            text="Edit",
            font=("Helvetica", 10),  # Keep original font size
            fg_color="#FFC107",
            hover_color="#FFD54F",
            text_color="#000",
            width=50,  # Smaller width
            height=25,  # Keep original height
            command=edit_product,
        )
        edit_btn.pack(side="left", padx=2)

        def delete_product():
            if messagebox.askyesno(
                "Confirm Delete", f"Delete product {product['name']}?"
            ):
                headers = {"Authorization": f"Bearer {token}"}
                try:
                    resp = requests.delete(
                        f"{API_URL}/product/delete/{product['id']}", headers=headers
                    )
                    if resp.status_code == 200:
                        messagebox.showinfo("Success", "Product deleted successfully")

                        # Refresh both the product list and the dashboard
                        fetch_products()  # Refresh the product list

                        # Schedule dashboard refresh
                        root = parent.winfo_toplevel()
                        root.after(300, lambda: switch_func("owner_dashboard", True))
                        root.after(400, lambda: switch_func("owner_products"))
                    elif resp.status_code == 400:
                        # Handle case where product is referenced in orders
                        try:
                            error_data = resp.json()
                            error_message = error_data.get(
                                "detail", "Cannot delete this product"
                            )

                            # Ask if user wants to archive the product instead
                            if messagebox.askyesno(
                                "Cannot Delete Product",
                                f"{error_message}\n\nWould you like to archive this product instead (set stock to 0)?",
                            ):
                                archive_product(product["id"])
                        except:
                            if messagebox.askyesno(
                                "Cannot Delete Product",
                                "This product cannot be deleted because it has been ordered by customers.\n\nWould you like to archive this product instead (set stock to 0)?",
                            ):
                                archive_product(product["id"])
                    else:
                        messagebox.showerror(
                            "Error", f"Failed to delete product: {resp.text}"
                        )
                except Exception as e:
                    messagebox.showerror("Error", f"Request error: {e}")

        def archive_product(product_id):
            """Set a product's stock to 0 to effectively archive it"""
            headers = {"Authorization": f"Bearer {token}"}

            # First get the current product data
            try:
                get_resp = requests.get(
                    f"{API_URL}/product/get/{product_id}", headers=headers
                )
                if get_resp.status_code == 200:
                    product_data = get_resp.json()

                    # Prepare the data for update, setting stock to 0
                    update_data = {
                        "name": product_data["name"],
                        "description": product_data["description"],
                        "price": product_data["price"],
                        "stock": 0,  # Set stock to 0
                        "shop_id": product_data["shop_id"],
                        "category_id": product_data["category_id"] or 1,
                    }

                    # Create an empty file for the images field
                    import io

                    empty_file = io.BytesIO(b"")
                    empty_file.name = "empty.jpg"
                    files = {"images": empty_file}

                    # Update the product
                    update_resp = requests.put(
                        f"{API_URL}/product/put/{product_id}",
                        data=update_data,
                        files=files,
                        headers=headers,
                    )

                    if update_resp.status_code == 200:
                        messagebox.showinfo(
                            "Success", "Product archived successfully (stock set to 0)"
                        )

                        # Refresh both the product list and the dashboard
                        fetch_products()  # Refresh the product list

                        # Schedule dashboard refresh
                        root = parent.winfo_toplevel()
                        root.after(300, lambda: switch_func("owner_dashboard", True))
                        root.after(400, lambda: switch_func("owner_products"))
                    else:
                        messagebox.showerror(
                            "Error", f"Failed to archive product: {update_resp.text}"
                        )
                else:
                    messagebox.showerror(
                        "Error", f"Failed to get product data: {get_resp.text}"
                    )
            except Exception as e:
                messagebox.showerror("Error", f"Request error: {e}")
            finally:
                if "files" in locals() and files and "images" in files:
                    files["images"].close()

        delete_btn = ctk.CTkButton(
            buttons_frame,
            text="Delete",
            font=("Helvetica", 10),  # Keep original font size
            fg_color="#F44336",
            hover_color="#EF5350",
            width=50,  # Smaller width
            height=25,  # Keep original height
            command=delete_product,
        )
        delete_btn.pack(side="left", padx=2)

        return row

    def clear_products_table():
        """Clear all product rows from the table"""
        for row in product_rows:
            row.destroy()
        product_rows.clear()

    def fetch_products():
        """Fetch shop and products data from the API"""
        headers = {"Authorization": f"Bearer {token}"}
        try:
            # First get the owner's shop
            shop_resp = requests.get(f"{API_URL}/shops/owner", headers=headers)
            if shop_resp.status_code == 200:
                shops = shop_resp.json()
                if shops:
                    nonlocal shop_data
                    shop_data = shops[0]  # Use the first shop

                    # Now fetch products for this shop
                    prod_resp = requests.get(
                        f"{API_URL}/product/shop/{shop_data['id']}", headers=headers
                    )
                    if prod_resp.status_code == 200:
                        products = prod_resp.json()

                        # Clear existing product rows
                        clear_products_table()

                        # Create new rows for each product
                        for product in products:
                            row = create_product_row(product)
                            product_rows.append(row)

                        if not products:
                            no_products_label = ctk.CTkLabel(
                                table_frame,
                                text="No products found. Add some products to your shop!",
                                font=("Helvetica", 14),
                                text_color="gray",
                            )
                            no_products_label.pack(pady=30)
                            product_rows.append(no_products_label)
                    else:
                        print(f"Error fetching products: {prod_resp.status_code}")
                        messagebox.showerror(
                            "Error", f"Failed to fetch products: {prod_resp.text}"
                        )
                else:
                    print("No shops found for this owner")
                    no_shop_label = ctk.CTkLabel(
                        table_frame,
                        text="You don't have any shops. Please create a shop first!",
                        font=("Helvetica", 14),
                        text_color="gray",
                    )
                    no_shop_label.pack(pady=30)
                    product_rows.append(no_shop_label)
            else:
                print(f"Error fetching shop: {shop_resp.status_code}")
                messagebox.showerror("Error", f"Failed to fetch shop: {shop_resp.text}")
        except Exception as e:
            print(f"Error in fetch_products: {e}")
            messagebox.showerror("Error", f"Request error: {e}")

    # Load products on initialization
    fetch_products()

    # Add refresh method to frame for external calls
    def refresh_data(*args):
        # Refresh products regardless of arguments passed
        fetch_products()

    frame.refresh_data = refresh_data

    return frame
