import customtkinter as ctk
from tkinter import filedialog, messagebox
import requests
import os
from PIL import Image, ImageTk

# Color scheme matching app theme
SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"
SIDEBAR_BG = "#2D3035"


def create_product_frame(parent, switch_func, API_URL, token):
    """
    Frame to create a new product.
    Fields: Title, Price, Description, Stock, and Product Image.
    On success, navigates back to the shop view.
    """
    # Main container with dark theme
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)

    def update_token(new_token):
        nonlocal token
        token = new_token

    frame.update_token = update_token  # Add method to the frame

    # --- TOP BAR ---
    top_bar = ctk.CTkFrame(frame, fg_color=SHOPPING, height=50)
    top_bar.pack(fill="x", side="top")

    top_label = ctk.CTkLabel(
        top_bar,
        text="Create New Product",
        text_color="white",
        font=("Helvetica", 20, "bold"),
    )
    top_label.pack(side="left", padx=25)

    def go_back():
        switch_func("owner_products")

    back_button = ctk.CTkButton(
        top_bar,
        text="← Back",
        fg_color="transparent",
        text_color="white",
        hover_color="#0096ff",
        command=go_back,
        width=80,
        height=35,
        font=("Helvetica", 14),
    )
    back_button.pack(side="right", padx=20, pady=7)

    # --- MAIN CONTENT ---
    main_content = ctk.CTkFrame(frame, fg_color="transparent")
    main_content.pack(fill="both", expand=True, padx=40, pady=30)

    # Form container with card styling
    form_card = ctk.CTkFrame(main_content, fg_color=CARD_BG, corner_radius=15)
    form_card.pack(fill="both", expand=True)

    form_inner = ctk.CTkFrame(form_card, fg_color="transparent")
    form_inner.pack(fill="both", expand=True, padx=30, pady=30)

    # Title and description
    header_frame = ctk.CTkFrame(form_inner, fg_color="transparent")
    header_frame.pack(fill="x", pady=(0, 20))

    title_label = ctk.CTkLabel(
        header_frame,
        text="Add Product Details",
        font=("Helvetica", 18, "bold"),
        text_color=SHOPPING,
    )
    title_label.pack(anchor="w")

    subtitle_label = ctk.CTkLabel(
        header_frame,
        text="Fill in the information below to add a new product",
        font=("Helvetica", 14),
        text_color="gray",
    )
    subtitle_label.pack(anchor="w", pady=(5, 0))

    # Two column layout
    columns_frame = ctk.CTkFrame(form_inner, fg_color="transparent")
    columns_frame.pack(fill="both", expand=True)

    # Left column (form fields)
    left_col = ctk.CTkFrame(columns_frame, fg_color="transparent")
    left_col.pack(side="left", fill="both", expand=True, padx=(0, 15))

    def create_form_field(parent, label_text, placeholder_text):
        field_frame = ctk.CTkFrame(parent, fg_color="transparent")
        field_frame.pack(fill="x", pady=(0, 15))

        label = ctk.CTkLabel(
            field_frame, text=label_text, font=("Helvetica", 14), text_color="white"
        )
        label.pack(anchor="w")

        entry = ctk.CTkEntry(
            field_frame,
            height=40,
            corner_radius=8,
            placeholder_text=placeholder_text,
            fg_color="#3b3b3b",
            border_color=SHOPPING,
            text_color="white",
        )
        entry.pack(fill="x", pady=(5, 0))
        return entry

    # Form Fields
    prod_title_entry = create_form_field(
        left_col, "Product Title", "Enter product title..."
    )
    price_entry = create_form_field(left_col, "Price ($)", "Enter price...")
    stock_entry = create_form_field(
        left_col, "Stock Quantity", "Enter available stock..."
    )

    # Description with multiline entry
    desc_frame = ctk.CTkFrame(left_col, fg_color="transparent")
    desc_frame.pack(fill="x", pady=(0, 15))

    desc_label = ctk.CTkLabel(
        desc_frame, text="Description", font=("Helvetica", 14), text_color="white"
    )
    desc_label.pack(anchor="w")

    desc_entry = ctk.CTkTextbox(
        desc_frame,
        height=100,
        corner_radius=8,
        fg_color="#3b3b3b",
        border_color=SHOPPING,
        text_color="white",
    )
    desc_entry.pack(fill="x", pady=(5, 0))

    # Right column (image upload)
    right_col = ctk.CTkFrame(columns_frame, fg_color="transparent")
    right_col.pack(side="left", fill="both", expand=True, padx=(15, 0))

    image_frame = ctk.CTkFrame(right_col, fg_color="transparent")
    image_frame.pack(fill="x", pady=(0, 15))

    image_label = ctk.CTkLabel(
        image_frame, text="Product Image", font=("Helvetica", 14), text_color="white"
    )
    image_label.pack(anchor="w")

    # Image preview area
    preview_frame = ctk.CTkFrame(
        image_frame, fg_color="#3b3b3b", corner_radius=10, height=200
    )
    preview_frame.pack(fill="x", pady=(5, 0))
    preview_frame.pack_propagate(False)

    preview_label = ctk.CTkLabel(
        preview_frame,
        text="No image selected",
        font=("Helvetica", 14),
        text_color="gray",
    )
    preview_label.pack(expand=True)

    # Track image path
    image_path = ctk.StringVar(value="")

    def browse_image():
        filename = filedialog.askopenfilename(
            title="Select Product Image",
            filetypes=[("Image Files", "*.png *.jpg *.jpeg *.gif")],
        )
        if filename:
            image_path.set(filename)
            try:
                pil_img = Image.open(filename).resize((150, 150))
                tk_img = ImageTk.PhotoImage(pil_img)
                preview_label.configure(image=tk_img, text="")
                preview_label.image = tk_img
            except Exception as e:
                print(f"Image preview error: {e}")

    browse_button = ctk.CTkButton(
        image_frame,
        text="Browse Image",
        command=browse_image,
        corner_radius=8,
        height=40,
        font=("Helvetica", 14),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    )
    browse_button.pack(pady=(10, 0))

    # Buttons at the bottom
    buttons_frame = ctk.CTkFrame(form_inner, fg_color="transparent")
    buttons_frame.pack(fill="x", pady=(20, 0))

    def submit_product():
        title_val = prod_title_entry.get().strip()
        price_val = price_entry.get().strip()
        desc_val = desc_entry.get("1.0", "end-1c").strip()
        stock_val = stock_entry.get().strip()

        if not title_val or not price_val or not stock_val:
            messagebox.showerror("Error", "Title, Price, and Stock are required")
            return

        # Validate that price and stock are positive numbers
        try:
            price = float(price_val)
            if price <= 0:
                messagebox.showerror("Invalid Price", "Price must be greater than zero")
                price_entry.delete(0, "end")
                price_entry.focus()
                return
        except ValueError:
            messagebox.showerror("Invalid Price", "Price must be a valid number")
            price_entry.delete(0, "end")
            price_entry.focus()
            return

        try:
            stock = int(stock_val)
            if stock <= 0:
                messagebox.showerror(
                    "Invalid Stock", "Stock quantity must be greater than zero"
                )
                stock_entry.delete(0, "end")
                stock_entry.focus()
                return
        except ValueError:
            messagebox.showerror(
                "Invalid Stock", "Stock quantity must be a valid integer"
            )
            stock_entry.delete(0, "end")
            stock_entry.focus()
            return

        # Get the owner's shop first
        headers = {"Authorization": f"Bearer {token}"}
        try:
            # Fetch the owner's shop
            shop_resp = requests.get(f"{API_URL}/shops/owner", headers=headers)
            if shop_resp.status_code != 200:
                messagebox.showerror(
                    "Error", f"Failed to get shop data: {shop_resp.text}"
                )
                return

            shops = shop_resp.json()
            if not shops:
                messagebox.showerror(
                    "Error", "You don't have any shops. Please create a shop first."
                )
                return

            shop_id = shops[0]["id"]  # Use the first shop's ID

            # Now create the product with the shop_id
            data = {
                "name": title_val,
                "price": price_val,
                "description": desc_val,
                "stock": stock_val,
                "shop_id": shop_id,  # Include the shop_id
            }
            files = {}
            if os.path.isfile(image_path.get()):
                files["images"] = open(image_path.get(), "rb")

            resp = requests.post(
                f"{API_URL}/product/create", data=data, files=files, headers=headers
            )
            if resp.status_code == 200:
                messagebox.showinfo("Success", "Product created successfully!")

                # Switch to owner_products and tell it to refresh
                switch_func("owner_products", True)

                # Wait a moment then switch to dashboard to refresh it too
                root = parent.winfo_toplevel()
                root.after(100, lambda: switch_func("owner_dashboard", True))
                root.after(200, lambda: switch_func("owner_products", True))
            else:
                messagebox.showerror(
                    "Error",
                    f"Failed to create product. Status: {resp.status_code}\n{resp.text}",
                )
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")
        finally:
            if files and "images" in files:
                files["images"].close()

    # Cancel button
    cancel_button = ctk.CTkButton(
        buttons_frame,
        text="Cancel",
        command=go_back,
        corner_radius=8,
        height=45,
        font=("Helvetica", 14),
        fg_color="#555555",
        hover_color="#777777",
    )
    cancel_button.pack(side="left", fill="x", expand=True, padx=(0, 5))

    # Create button
    submit_button = ctk.CTkButton(
        buttons_frame,
        text="Create Product",
        command=submit_product,
        corner_radius=8,
        height=45,
        font=("Helvetica", 14, "bold"),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    )
    submit_button.pack(side="right", fill="x", expand=True, padx=(5, 0))

    return frame
