import customtkinter as ctk
import requests
from PIL import Image, ImageTk
from tkinter import messagebox, filedialog
import io

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"


def user_details_frame(parent, switch_func, API_URL, token):
    """
    User details page with a modern dark theme and improved layout.
    """
    # Main container frame
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)
    frame.token = token  # Store token as an attribute
    is_owner = False  # Flag to track if user is a shop owner

    def update_token(new_token):
        """Update the token when it changes"""
        frame.token = new_token

    # Add the update_token method to the frame
    frame.update_token = update_token

    # --- TOP BAR ---
    top_bar = ctk.CTkFrame(frame, fg_color=SHOPPING, height=50)
    top_bar.pack(fill="x", side="top")

    top_label = ctk.CTkLabel(
        top_bar, text="My Profile", text_color="white", font=("Helvetica", 20, "bold")
    )
    top_label.pack(side="left", padx=25)

    def go_back():
        # For shop owners, use dashboard_bypass to ensure they go to the user dashboard
        if is_owner:
            switch_func("dashboard_bypass", frame.token)
        else:
            switch_func("dashboard")

    back_button = ctk.CTkButton(
        top_bar,
        text="← Back",
        fg_color="transparent",
        text_color="white",
        hover_color="#0096ff",
        command=go_back,
        width=80,
        height=35,
        font=("Helvetica", 14),
    )
    back_button.pack(side="right", padx=20, pady=7)

    # --- MAIN SECTION: Sidebar + Content ---
    main_section = ctk.CTkFrame(frame, fg_color="transparent")
    main_section.pack(fill="both", expand=True)

    # LEFT SIDEBAR
    sidebar_frame = ctk.CTkFrame(
        main_section, width=250, fg_color=CARD_BG, corner_radius=15
    )
    sidebar_frame.pack(side="left", fill="y", padx=20, pady=20)
    sidebar_frame.pack_propagate(False)

    sidebar_title = ctk.CTkLabel(
        sidebar_frame, text="Menu", font=("Helvetica", 18, "bold"), text_color=SHOPPING
    )
    sidebar_title.pack(pady=(20, 10))

    def create_nav_button(text, command=None, is_active=False):
        return ctk.CTkButton(
            sidebar_frame,
            text=text,
            fg_color="transparent",
            text_color="white",
            hover_color="#3b3b3b",
            command=command,
            height=40,
            font=("Helvetica", 14),
            state="disabled" if is_active else "normal",
        )

    def dashboard_nav():
        # For shop owners, use dashboard_bypass to ensure they go to the user dashboard
        if is_owner:
            switch_func("dashboard_bypass", frame.token)
        else:
            switch_func("dashboard")

    nav_dashboard = create_nav_button("Dashboard", dashboard_nav)
    nav_dashboard.pack(fill="x", padx=15, pady=5)

    nav_profile = create_nav_button("My Profile", is_active=True)
    nav_profile.pack(fill="x", padx=15, pady=5)

    def open_orders():
        switch_func("user_orders")

    nav_orders = create_nav_button("My Orders", open_orders)
    nav_orders.pack(fill="x", padx=15, pady=5)

    def open_payments():
        switch_func("user_payments")

    nav_payments = create_nav_button("My Payments", open_payments)
    nav_payments.pack(fill="x", padx=15, pady=5)

    # Create a variable for the owner button that can be modified later
    owner_button_frame = ctk.CTkFrame(sidebar_frame, fg_color="transparent")
    # Pack with minimal padding to be closer to nav buttons
    owner_button_frame.pack(fill="x", padx=15, pady=0)

    def open_create_shop():
        switch_func("create_shop")

    def go_to_owner_dashboard():
        switch_func("owner_dashboard")

    # The button will be created after we check the user's role
    owner_button = None

    # Add a separator before logout
    separator = ctk.CTkFrame(sidebar_frame, fg_color="#3b3b3b", height=1)
    separator.pack(fill="x", padx=15, pady=10, side="bottom")

    # Add logout button with red accent
    def logout():
        switch_func("login")

    logout_button = ctk.CTkButton(
        sidebar_frame,
        text="Log Out",
        fg_color="#ff4242",  # Red color
        text_color="white",
        hover_color="#ff6b6b",
        command=logout,
        height=40,
        font=("Helvetica", 14, "bold"),
    )
    logout_button.pack(fill="x", padx=15, pady=5, side="bottom")  # Place at bottom

    # RIGHT CONTENT (User Details)
    content_frame = ctk.CTkFrame(main_section, fg_color=CARD_BG, corner_radius=15)
    content_frame.pack(side="left", fill="both", expand=True, padx=(0, 20), pady=20)

    # Header
    header_frame = ctk.CTkFrame(content_frame, fg_color="transparent")
    header_frame.pack(fill="x", padx=30, pady=20)

    title_label = ctk.CTkLabel(
        header_frame,
        text="Profile Information",
        font=("Helvetica", 24, "bold"),
        text_color=SHOPPING,
    )
    title_label.pack(anchor="w")

    subtitle_label = ctk.CTkLabel(
        header_frame,
        text="View and manage your profile information",
        font=("Helvetica", 14),
        text_color="gray",
    )
    subtitle_label.pack(anchor="w", pady=(5, 0))

    # Main Content
    main_content = ctk.CTkFrame(content_frame, fg_color="transparent")
    main_content.pack(fill="both", expand=True, padx=30, pady=(0, 20))

    # View Mode Frame (initial view)
    view_frame = ctk.CTkFrame(main_content, fg_color="transparent")
    view_frame.pack(fill="both", expand=True)

    # Edit Mode Frame (hidden initially)
    edit_frame = ctk.CTkFrame(main_content, fg_color="transparent")
    # Not packing this yet - it will be shown when edit mode is activated

    # --- VIEW MODE UI ---
    view_content = ctk.CTkFrame(view_frame, fg_color="transparent")
    view_content.pack(fill="both", expand=True, padx=30)

    # Function to create a user detail row in view mode
    def create_detail_row(parent, label_text, value_text=""):
        row = ctk.CTkFrame(parent, fg_color="transparent", height=50)
        row.pack(fill="x", pady=(0, 15))
        row.pack_propagate(False)

        label = ctk.CTkLabel(
            row,
            text=label_text,
            font=("Helvetica", 14, "bold"),
            text_color="white",
            width=120,
        )
        label.pack(side="left", anchor="w")

        value = ctk.CTkLabel(
            row, text=value_text, font=("Helvetica", 14), text_color="white"
        )
        value.pack(side="left", padx=(20, 0), anchor="w")

        return value  # Return so we can update it later

    # Create view mode fields
    username_view = create_detail_row(view_content, "Username:")
    email_view = create_detail_row(view_content, "Email:")
    phone_view = create_detail_row(view_content, "Phone:")

    # Button container for view mode
    button_container_view = ctk.CTkFrame(view_content, fg_color="transparent")
    button_container_view.pack(fill="x", pady=(20, 0))

    # --- EDIT MODE UI ---
    # Edit Form
    form_frame = ctk.CTkFrame(edit_frame, fg_color="transparent")
    form_frame.pack(fill="both", expand=True, padx=30)

    def create_form_field(label_text, placeholder_text):
        field_frame = ctk.CTkFrame(form_frame, fg_color="transparent")
        field_frame.pack(fill="x", pady=(0, 15))

        label = ctk.CTkLabel(
            field_frame, text=label_text, font=("Helvetica", 14), text_color="white"
        )
        label.pack(anchor="w")

        entry = ctk.CTkEntry(
            field_frame,
            height=40,
            corner_radius=8,
            placeholder_text=placeholder_text,
            fg_color="#3b3b3b",
            border_color=SHOPPING,
            text_color="white",
        )
        entry.pack(fill="x", pady=(5, 0))
        return entry

    username_entry = create_form_field("Username", "Enter your username...")
    email_entry = create_form_field("Email", "Enter your email...")
    phone_entry = create_form_field("Phone", "Enter your phone number...")

    # Button container for save/cancel in edit mode
    button_container_edit = ctk.CTkFrame(form_frame, fg_color="transparent")
    button_container_edit.pack(fill="x", pady=(20, 0))

    # Mode switching functions
    def enable_edit_mode():
        view_frame.pack_forget()
        edit_frame.pack(fill="both", expand=True)

        # Copy data from view to edit mode fields
        username_entry.delete(0, "end")
        username_entry.insert(0, username_view.cget("text"))

        email_entry.delete(0, "end")
        email_entry.insert(0, email_view.cget("text"))
        email_entry.configure(state="disabled")  # Make email field read-only

        phone_entry.delete(0, "end")
        phone_entry.insert(0, phone_view.cget("text"))

    def cancel_edit():
        edit_frame.pack_forget()
        view_frame.pack(fill="both", expand=True)
        email_entry.configure(state="normal")  # Reset state when canceling

    # Edit button in view mode
    edit_button = ctk.CTkButton(
        button_container_view,
        text="Edit Profile",
        command=enable_edit_mode,
        corner_radius=8,
        height=40,
        font=("Helvetica", 14, "bold"),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    )
    edit_button.pack(fill="x")

    # Save and Cancel buttons in edit mode
    def save_profile():
        headers = {"Authorization": f"Bearer {frame.token}"}
        payload = {
            "username": username_entry.get().strip(),
            "phone": phone_entry.get().strip(),  # Note: backend expects "phone", not "phone_number"
            # Email is intentionally omitted from the payload to prevent changes
        }
        try:
            resp = requests.put(f"{API_URL}/user/update", headers=headers, json=payload)
            if resp.status_code == 200:
                messagebox.showinfo("Success", "Profile updated successfully!")

                # Update the view labels with new data
                username_view.configure(text=payload["username"])
                phone_view.configure(text=payload["phone"])

                # Switch back to view mode
                cancel_edit()
            else:
                messagebox.showerror("Error", "Unable to update profile.")
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    save_button = ctk.CTkButton(
        button_container_edit,
        text="Save Changes",
        command=save_profile,
        corner_radius=8,
        height=45,
        font=("Helvetica", 14, "bold"),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    )
    save_button.pack(side="left", fill="x", expand=True, padx=(0, 5))

    cancel_button = ctk.CTkButton(
        button_container_edit,
        text="Cancel",
        command=cancel_edit,
        corner_radius=8,
        height=45,
        font=("Helvetica", 14),
        fg_color="#555555",
        hover_color="#777777",
    )
    cancel_button.pack(side="right", fill="x", expand=True, padx=(5, 0))

    def fetch_user_info():
        headers = {"Authorization": f"Bearer {frame.token}"}
        print(
            f"Fetching user info with token: {frame.token[:10] if frame.token else 'None'}"
        )
        try:
            resp = requests.get(f"{API_URL}/user/profile", headers=headers)
            print(f"User profile response: {resp.status_code}")
            if resp.status_code == 200:
                data = resp.json()
                print(f"User data received: {data}")

                # Update view mode displays
                username_view.configure(text=data.get("username", ""))
                email_view.configure(text=data.get("email", ""))
                phone_view.configure(text=data.get("phone", ""))

                # Check if user is an owner and update UI accordingly
                nonlocal is_owner, owner_button
                user_role = data.get("role", "")
                is_owner = user_role == "owner" or user_role == "shop_owner"

                # Remove any existing button in the owner button frame
                for widget in owner_button_frame.winfo_children():
                    widget.destroy()

                # Create the appropriate button based on user role
                if is_owner:
                    # Create a button with the same styling as other nav buttons
                    owner_button = ctk.CTkButton(
                        owner_button_frame,
                        text="Go to Shop Dashboard",
                        fg_color=SHOPPING,  # Use the shopping color for emphasis
                        text_color="white",
                        hover_color="#0096ff",
                        height=40,
                        command=go_to_owner_dashboard,
                        font=("Helvetica", 14, "bold"),
                    )
                else:
                    # Create a styled button that stands out better than the nav buttons
                    owner_button = ctk.CTkButton(
                        owner_button_frame,
                        text="Become Shop Owner",
                        command=open_create_shop,
                        fg_color="#3b3b3b",  # Darker than transparent but not too prominent
                        text_color="white",
                        hover_color="#4a4a4a",
                        height=40,
                        font=("Helvetica", 14),
                    )

                # Pack without any padding to match other buttons
                owner_button.pack(fill="x")

            else:
                print(f"Error response: {resp.text}")
                messagebox.showerror("Error", "Unable to retrieve user information.")
        except Exception as e:
            print(f"Exception in fetch_user_info: {e}")
            messagebox.showerror("Error", f"Request error: {e}")

    # Initialize by fetching user info
    fetch_user_info()

    return frame
