import customtkinter as ctk
import requests
from PIL import Image, ImageTk
from tkinter import messagebox
import io

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"
BACKEND_HOST = "http://127.0.0.1:8000"


def fix_url(url):
    if url.startswith("http"):
        return url
    prefix = "app/static/"
    if url.startswith(prefix):
        url = url[len(prefix) :]
    return f"{BACKEND_HOST}/static/{url}"


def user_orders_frame(parent, switch_func, API_URL, token):
    """
    A modern user orders page that displays the products the user has purchased
    along with associated shop details.
    """
    # Main container frame with dark background
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)
    frame.token = token  # Store token as an attribute
    is_owner = False  # Flag to track if user is a shop owner

    def update_token(new_token):
        """Update the token when it changes"""
        frame.token = new_token

    # Add the update_token method to the frame
    frame.update_token = update_token

    # --- TOP BAR ---
    top_bar = ctk.CTkFrame(frame, fg_color=SHOPPING, height=50)
    top_bar.pack(fill="x", side="top")

    top_label = ctk.CTkLabel(
        top_bar, text="My Orders", text_color="white", font=("Helvetica", 20, "bold")
    )
    top_label.pack(side="left", padx=25)

    def go_back():
        # For shop owners, use dashboard_bypass to ensure they go to the user dashboard
        if is_owner:
            switch_func("dashboard_bypass", frame.token)
        else:
            switch_func("dashboard")

    back_button = ctk.CTkButton(
        top_bar,
        text="← Back",
        fg_color="transparent",
        text_color="white",
        hover_color="#0096ff",
        command=go_back,
        width=80,
        height=35,
        font=("Helvetica", 14),
    )
    back_button.pack(side="right", padx=20, pady=7)

    # --- MAIN SECTION: Sidebar + Content ---
    main_section = ctk.CTkFrame(frame, fg_color="transparent")
    main_section.pack(fill="both", expand=True)

    # LEFT SIDEBAR
    sidebar_frame = ctk.CTkFrame(
        main_section, width=250, fg_color=CARD_BG, corner_radius=15
    )
    sidebar_frame.pack(side="left", fill="y", padx=20, pady=20)
    sidebar_frame.pack_propagate(False)

    sidebar_title = ctk.CTkLabel(
        sidebar_frame, text="Menu", font=("Helvetica", 18, "bold"), text_color=SHOPPING
    )
    sidebar_title.pack(pady=(20, 10))

    def create_nav_button(text, command=None, is_active=False):
        return ctk.CTkButton(
            sidebar_frame,
            text=text,
            fg_color="transparent",
            text_color="white",
            hover_color="#3b3b3b",
            command=command,
            height=40,
            font=("Helvetica", 14),
            state="disabled" if is_active else "normal",
        )

    def dashboard_nav():
        # For shop owners, use dashboard_bypass to ensure they go to the user dashboard
        if is_owner:
            switch_func("dashboard_bypass", frame.token)
        else:
            switch_func("dashboard")

    nav_dashboard = create_nav_button("Dashboard", dashboard_nav)
    nav_dashboard.pack(fill="x", padx=15, pady=5)

    def open_profile():
        switch_func("user_details")

    nav_profile = create_nav_button("My Profile", open_profile)
    nav_profile.pack(fill="x", padx=15, pady=5)

    nav_orders = create_nav_button("My Orders", is_active=True)
    nav_orders.pack(fill="x", padx=15, pady=5)

    def open_payments():
        switch_func("user_payments")

    nav_payments = create_nav_button("My Payments", open_payments)
    nav_payments.pack(fill="x", padx=15, pady=5)

    # Create a variable for the owner button that can be modified later
    owner_button_frame = ctk.CTkFrame(sidebar_frame, fg_color="transparent")
    # Pack with minimal padding to be closer to nav buttons
    owner_button_frame.pack(fill="x", padx=15, pady=0)

    def open_create_shop():
        switch_func("create_shop")

    def go_to_owner_dashboard():
        switch_func("owner_dashboard")

    # The button will be created after we check the user's role
    owner_button = None

    # Add a separator before logout
    separator = ctk.CTkFrame(sidebar_frame, fg_color="#3b3b3b", height=1)
    separator.pack(fill="x", padx=15, pady=10, side="bottom")

    # Add logout button with red accent
    def logout():
        switch_func("login")

    logout_button = ctk.CTkButton(
        sidebar_frame,
        text="Log Out",
        fg_color="#ff4242",  # Red color
        text_color="white",
        hover_color="#ff6b6b",
        command=logout,
        height=40,
        font=("Helvetica", 14, "bold"),
    )
    logout_button.pack(fill="x", padx=15, pady=5, side="bottom")  # Place at bottom

    # RIGHT CONTENT
    content_frame = ctk.CTkFrame(main_section, fg_color=CARD_BG, corner_radius=15)
    content_frame.pack(side="left", fill="both", expand=True, padx=(0, 20), pady=20)

    # Tab control for Cart and Order History
    tab_view = ctk.CTkTabview(content_frame, fg_color="transparent")
    tab_view.pack(fill="both", expand=True, padx=10, pady=10)

    # Create tabs
    cart_tab = tab_view.add("My Cart")
    orders_tab = tab_view.add("Order History")
    shipped_tab = tab_view.add("Done Shipping")

    # CART TAB
    cart_frame = ctk.CTkScrollableFrame(cart_tab, fg_color="transparent")
    cart_frame.pack(fill="both", expand=True, padx=20, pady=20)

    # Cart title with refresh button
    title_frame = ctk.CTkFrame(cart_frame, fg_color="transparent")
    title_frame.pack(fill="x", pady=(0, 20))

    cart_title = ctk.CTkLabel(
        title_frame,
        text="Your Shopping Cart",
        font=("Helvetica", 18, "bold"),
        text_color=SHOPPING,
    )
    cart_title.pack(side="left", anchor="w")

    def refresh_cart():
        """Function to reload cart items"""
        load_cart_items()

    refresh_cart_btn = ctk.CTkButton(
        title_frame,
        text="🔄 Refresh",
        font=("Helvetica", 12),
        fg_color="#4A4A4A",
        hover_color="#5A5A5A",
        width=100,
        height=28,
        command=refresh_cart,
    )
    refresh_cart_btn.pack(side="right", padx=5)

    cart_items_frame = ctk.CTkFrame(cart_frame, fg_color="transparent")
    cart_items_frame.pack(fill="both", expand=True)

    # Checkout frame (initially empty)
    checkout_frame = ctk.CTkFrame(cart_frame, fg_color="transparent")
    checkout_frame.pack(fill="x", pady=10)

    def load_cart_items():
        # Clear existing items
        for widget in cart_items_frame.winfo_children():
            widget.destroy()

        # Clear checkout frame
        for widget in checkout_frame.winfo_children():
            widget.destroy()

        try:
            headers = {"Authorization": f"Bearer {frame.token}"}
            response = requests.get(f"{API_URL}/cart/items", headers=headers)

            if response.status_code == 200:
                cart_data = response.json()
                cart_items = cart_data.get("items", [])
                total_price = cart_data.get("total_price", 0)

                if not cart_items:
                    empty_label = ctk.CTkLabel(
                        cart_items_frame,
                        text="Your cart is empty",
                        font=("Helvetica", 14),
                    )
                    empty_label.pack(pady=20)
                    return

                # Header row
                header_frame = ctk.CTkFrame(cart_items_frame, fg_color="#333333")
                header_frame.pack(fill="x", pady=(0, 10))

                # Configure the same grid layout as the items
                header_frame.grid_columnconfigure(
                    0, weight=0, minsize=40
                )  # Checkbox column
                header_frame.grid_columnconfigure(
                    1, weight=1, minsize=300
                )  # Product column (expandable)
                header_frame.grid_columnconfigure(
                    2, weight=0, minsize=100
                )  # Unit price column
                header_frame.grid_columnconfigure(
                    3, weight=0, minsize=100
                )  # Quantity column
                header_frame.grid_columnconfigure(
                    4, weight=0, minsize=100
                )  # Subtotal column
                header_frame.grid_columnconfigure(
                    5, weight=0, minsize=90
                )  # Actions column

                # Checkbox header (select all)
                select_all_var = ctk.BooleanVar(value=False)
                selected_items = {}  # Dictionary to track selected items {item_id: bool}

                def toggle_select_all():
                    for item_id, checkbox_var in selected_items.items():
                        checkbox_var.set(select_all_var.get())

                select_all_cb = ctk.CTkCheckBox(
                    header_frame,
                    text="",
                    variable=select_all_var,
                    command=toggle_select_all,
                    width=30,
                )
                select_all_cb.grid(row=0, column=0, padx=5, pady=10, sticky="w")

                ctk.CTkLabel(
                    header_frame,
                    text="Product",
                    font=("Helvetica", 12, "bold"),
                ).grid(row=0, column=1, padx=5, pady=10, sticky="w")

                ctk.CTkLabel(
                    header_frame,
                    text="Unit Price",
                    font=("Helvetica", 12, "bold"),
                ).grid(row=0, column=2, padx=5, pady=10)

                ctk.CTkLabel(
                    header_frame,
                    text="Quantity",
                    font=("Helvetica", 12, "bold"),
                ).grid(row=0, column=3, padx=5, pady=10)

                ctk.CTkLabel(
                    header_frame,
                    text="Subtotal",
                    font=("Helvetica", 12, "bold"),
                ).grid(row=0, column=4, padx=5, pady=10)

                ctk.CTkLabel(
                    header_frame,
                    text="Actions",
                    font=("Helvetica", 12, "bold"),
                ).grid(row=0, column=5, padx=5, pady=10)

                # Create items list
                for item in cart_items:
                    item_id = item.get("id")
                    # Create checkbox variable for this item
                    selected_items[item_id] = ctk.BooleanVar(value=False)

                    item_frame = create_cart_item_frame(item, selected_items[item_id])
                    item_frame.pack(fill="x", pady=5)

                # Add checkout section
                divider = ctk.CTkFrame(checkout_frame, fg_color="#444444", height=1)
                divider.pack(fill="x", pady=10)

                # Selected items summary
                def update_checkout_summary():
                    # Calculate selected items total
                    selected_total = 0
                    selected_count = 0

                    for item in cart_items:
                        item_id = item.get("id")
                        if item_id in selected_items and selected_items[item_id].get():
                            selected_total += item.get("subtotal", 0)
                            selected_count += 1

                    # Update summary label
                    summary_label.configure(
                        text=f"Selected Items: {selected_count} | Total: ${selected_total:.2f}"
                    )

                    # Update checkout button state
                    if selected_count > 0:
                        checkout_button.configure(state="normal")
                    else:
                        checkout_button.configure(state="disabled")

                # Create a frame for the summary
                bottom_frame = ctk.CTkFrame(checkout_frame, fg_color="transparent")
                bottom_frame.pack(fill="x", pady=15)

                # Create the summary label
                summary_label = ctk.CTkLabel(
                    bottom_frame,
                    text="Selected Items: 0 | Total: $0.00",
                    font=("Helvetica", 14, "bold"),
                    text_color=SHOPPING,
                )
                summary_label.pack(side="left", padx=10)

                # Function to update selection count (bound to each checkbox)
                def on_checkbox_change():
                    update_checkout_summary()

                    # Check if all are selected
                    all_selected = all(var.get() for var in selected_items.values())
                    select_all_var.set(all_selected)

                # Bind the change function to all checkboxes
                for var in selected_items.values():
                    var.trace_add("write", lambda *args: on_checkbox_change())

                # Delivery address
                address_frame = ctk.CTkFrame(checkout_frame, fg_color="transparent")
                address_frame.pack(fill="x", pady=10)

                address_label = ctk.CTkLabel(
                    address_frame,
                    text="Delivery Address:",
                    font=("Helvetica", 14),
                    text_color="#AAAAAA",
                )
                address_label.pack(side="left", padx=10)

                address_entry = ctk.CTkEntry(address_frame, width=300)
                address_entry.pack(side="left", padx=10)
                address_entry.insert(
                    0, "10 Downing Street, London, UK"
                )  # Default address

                # Payment selection
                payment_frame = ctk.CTkFrame(checkout_frame, fg_color="transparent")
                payment_frame.pack(fill="x", pady=10)

                payment_label = ctk.CTkLabel(
                    payment_frame,
                    text="Payment Method:",
                    font=("Helvetica", 14),
                    text_color="#AAAAAA",
                )
                payment_label.pack(side="left", padx=10)

                # Function to load payment methods
                def load_payment_methods():
                    payment_options.clear()
                    payment_ids.clear()

                    # Get payment methods
                    try:
                        payments_response = requests.get(
                            f"{API_URL}/payment/user", headers=headers
                        )

                        if payments_response.status_code == 200:
                            payments = payments_response.json()
                            for payment in payments:
                                payment_text = f"{payment.get('payment_method')} ending in {payment.get('card_number')[-4:]}"
                                payment_options.append(payment_text)
                                payment_ids[payment_text] = payment.get("id")

                        if not payment_options:
                            payment_options.append("No payment methods available")

                        # Update dropdown
                        payment_dropdown.configure(values=payment_options)
                        payment_var.set(payment_options[0] if payment_options else "")
                    except Exception as e:
                        print(f"Error loading payment methods: {e}")

                # Get payment methods
                payment_options = []
                payment_ids = {}

                # Create a frame to hold the dropdown and refresh button
                payment_dropdown_frame = ctk.CTkFrame(
                    payment_frame, fg_color="transparent"
                )
                payment_dropdown_frame.pack(side="left", fill="x", expand=True)

                payment_var = ctk.StringVar(value="")
                payment_dropdown = ctk.CTkOptionMenu(
                    payment_dropdown_frame,
                    values=["Loading..."],
                    variable=payment_var,
                    width=250,
                )
                payment_dropdown.pack(side="left", padx=10)

                # Refresh button
                refresh_btn = ctk.CTkButton(
                    payment_dropdown_frame,
                    text="🔄",
                    width=40,
                    height=28,
                    command=load_payment_methods,
                    fg_color="#4A4A4A",
                    hover_color="#5A5A5A",
                )
                refresh_btn.pack(side="left", padx=5)

                # Load payment methods initially
                load_payment_methods()

                def add_payment_method():
                    """Function to redirect to payment methods page"""
                    switch_func("user_payments")

                add_payment_btn = ctk.CTkButton(
                    payment_frame,
                    text="Add Payment Method",
                    font=("Helvetica", 12),
                    fg_color="#5A5A5A",
                    hover_color="#6A6A6A",
                    command=add_payment_method,
                    width=150,
                )
                add_payment_btn.pack(side="left", padx=10)

                # Checkout button
                def handle_checkout():
                    # Get selected item IDs
                    selected_item_ids = [
                        item_id for item_id, var in selected_items.items() if var.get()
                    ]

                    if not selected_item_ids:
                        messagebox.showerror("Error", "Please select at least one item")
                        return

                    if (
                        not payment_options
                        or payment_options[0] == "No payment methods available"
                    ):
                        result = messagebox.askyesno(
                            "No Payment Method",
                            "You don't have any payment methods. Would you like to add one now?",
                        )
                        if result:
                            switch_func("user_payments")
                        return

                    delivery_addr = address_entry.get().strip()
                    if not delivery_addr:
                        messagebox.showerror("Error", "Please enter a delivery address")
                        return

                    selected_payment = payment_var.get()
                    payment_id = payment_ids.get(selected_payment)

                    if not payment_id:
                        messagebox.showerror("Error", "Invalid payment method selected")
                        return

                    try:
                        checkout_response = requests.post(
                            f"{API_URL}/cart/checkout",
                            json={
                                "delivery_address": delivery_addr,
                                "payment_id": payment_id,
                                "selected_items": selected_item_ids,
                            },
                            headers=headers,
                        )

                        if checkout_response.status_code == 200:
                            messagebox.showinfo(
                                "Success", "Your order has been placed successfully!"
                            )
                            # Refresh order history and clear cart view
                            load_order_history()
                            load_cart_items()
                            # Switch to order history tab
                            tab_view.set("Order History")
                        else:
                            error_msg = checkout_response.json().get(
                                "detail", "Unknown error"
                            )
                            messagebox.showerror(
                                "Checkout Failed", f"Error: {error_msg}"
                            )

                    except Exception as e:
                        messagebox.showerror("Error", f"Checkout failed: {str(e)}")

                checkout_button = ctk.CTkButton(
                    bottom_frame,
                    text="Checkout Selected Items",
                    font=("Helvetica", 14, "bold"),
                    fg_color="#ff4242",
                    hover_color="#ff6b6b",
                    command=handle_checkout,
                    state="disabled",  # Initially disabled until items are selected
                    width=200,
                    height=40,
                )
                checkout_button.pack(side="right", padx=15)

            else:
                error_msg = response.json().get("detail", "Unknown error")
                messagebox.showerror("Error", f"Failed to load cart: {error_msg}")

        except Exception as e:
            messagebox.showerror("Error", f"Failed to load cart: {str(e)}")

    def create_cart_item_frame(item, checkbox_var):
        item_frame = ctk.CTkFrame(cart_items_frame, fg_color="#2b2b2b", corner_radius=5)

        # Create content frame - use grid layout for better alignment
        content = ctk.CTkFrame(item_frame, fg_color="transparent")
        content.pack(fill="x", padx=5, pady=10)

        # Configure grid columns with consistent widths
        content.grid_columnconfigure(0, weight=0, minsize=40)  # Checkbox column
        content.grid_columnconfigure(
            1, weight=1, minsize=300
        )  # Product column (expandable)
        content.grid_columnconfigure(2, weight=0, minsize=100)  # Unit price column
        content.grid_columnconfigure(3, weight=0, minsize=100)  # Quantity column
        content.grid_columnconfigure(4, weight=0, minsize=100)  # Subtotal column
        content.grid_columnconfigure(5, weight=0, minsize=90)  # Actions column

        # Checkbox - column 0
        item_checkbox = ctk.CTkCheckBox(
            content, text="", variable=checkbox_var, width=30
        )
        item_checkbox.grid(row=0, column=0, padx=5, sticky="w")

        # Product image and name - column 1
        product_frame = ctk.CTkFrame(content, fg_color="transparent")
        product_frame.grid(row=0, column=1, padx=5, sticky="w")

        # Create horizontal layout for image and text
        img_frame = ctk.CTkFrame(product_frame, fg_color="#333333", width=60, height=60)
        img_frame.pack(side="left", padx=5, pady=5)
        img_frame.pack_propagate(False)

        img_label = ctk.CTkLabel(img_frame, text="")
        img_label.place(relx=0.5, rely=0.5, anchor="center")

        # Try to load product image
        images = item.get("images", [])
        if images:
            img_url = images[0].get("image_url")
            if img_url:
                fixed_url = fix_url(img_url)
                try:
                    resp = requests.get(fixed_url)
                    if resp.status_code == 200:
                        pil_img = Image.open(io.BytesIO(resp.content)).resize((50, 50))
                        tk_img = ImageTk.PhotoImage(pil_img)
                        img_label.configure(image=tk_img, text="")
                        img_label.image = tk_img
                except Exception as e:
                    print(f"Failed to load product image: {e}")

        # Product details
        details_frame = ctk.CTkFrame(product_frame, fg_color="transparent")
        details_frame.pack(side="left", fill="both", expand=True, padx=10)

        # Product name
        product_name = ctk.CTkLabel(
            details_frame,
            text=item.get("name", "Unknown Product"),
            font=("Helvetica", 12, "bold"),
            justify="left",
            anchor="w",
        )
        product_name.pack(anchor="w")

        # Shop name
        shop_name = ctk.CTkLabel(
            details_frame,
            text=f"Shop: {item.get('shop_name', 'Unknown Shop')}",
            font=("Helvetica", 10),
            text_color="#AAAAAA",
            justify="left",
            anchor="w",
        )
        shop_name.pack(anchor="w")

        # Unit price - column 2
        unit_price = item.get("price", 0)
        price_label = ctk.CTkLabel(
            content,
            text=f"${unit_price:.2f}",
            font=("Helvetica", 12),
        )
        price_label.grid(row=0, column=2, padx=5)

        # Quantity adjustment - column 3
        quantity_frame = ctk.CTkFrame(content, fg_color="#1f1f1f")
        quantity_frame.grid(row=0, column=3, padx=5)

        quantity = item.get("quantity", 1)

        def update_quantity(delta):
            nonlocal quantity
            item_id = item.get("id")
            new_quantity = quantity + delta

            if 1 <= new_quantity <= 99:
                try:
                    headers = {"Authorization": f"Bearer {frame.token}"}
                    response = requests.put(
                        f"{API_URL}/cart/update/{item_id}",
                        json={"quantity": new_quantity},
                        headers=headers,
                    )

                    if response.status_code == 200:
                        # Update display
                        quantity = new_quantity
                        quantity_label.configure(text=str(quantity))

                        # Update subtotal
                        subtotal = unit_price * quantity
                        subtotal_label.configure(text=f"${subtotal:.2f}")

                        # Reload cart to refresh totals
                        load_cart_items()
                    else:
                        error_msg = response.json().get("detail", "Unknown error")
                        messagebox.showerror(
                            "Error", f"Failed to update quantity: {error_msg}"
                        )

                except Exception as e:
                    messagebox.showerror(
                        "Error", f"Failed to update quantity: {str(e)}"
                    )

        minus_btn = ctk.CTkButton(
            quantity_frame,
            text="-",
            width=25,
            height=25,
            font=("Helvetica", 12),
            command=lambda: update_quantity(-1),
        )
        minus_btn.pack(side="left", padx=2, pady=2)

        quantity_label = ctk.CTkLabel(
            quantity_frame,
            text=str(quantity),
            width=30,
            font=("Helvetica", 12),
        )
        quantity_label.pack(side="left")

        plus_btn = ctk.CTkButton(
            quantity_frame,
            text="+",
            width=25,
            height=25,
            font=("Helvetica", 12),
            command=lambda: update_quantity(1),
        )
        plus_btn.pack(side="left", padx=2, pady=2)

        # Subtotal - column 4
        subtotal = item.get("subtotal", 0)
        subtotal_label = ctk.CTkLabel(
            content,
            text=f"${subtotal:.2f}",
            font=("Helvetica", 12, "bold"),
            text_color=SHOPPING,
        )
        subtotal_label.grid(row=0, column=4, padx=5)

        # Actions - column 5
        def remove_item():
            try:
                headers = {"Authorization": f"Bearer {frame.token}"}
                response = requests.delete(
                    f"{API_URL}/cart/remove/{item.get('id')}", headers=headers
                )

                if response.status_code == 200:
                    load_cart_items()  # Refresh cart
                else:
                    error_msg = response.json().get("detail", "Unknown error")
                    messagebox.showerror("Error", f"Failed to remove item: {error_msg}")

            except Exception as e:
                messagebox.showerror("Error", f"Failed to remove item: {str(e)}")

        remove_btn = ctk.CTkButton(
            content,
            text="Remove",
            font=("Helvetica", 12),
            fg_color="#ff4242",
            hover_color="#ff6b6b",
            width=70,
            command=remove_item,
        )
        remove_btn.grid(row=0, column=5, padx=5)

        return item_frame

    # ORDER HISTORY TAB
    orders_frame = ctk.CTkScrollableFrame(orders_tab, fg_color="transparent")
    orders_frame.pack(fill="both", expand=True, padx=20, pady=20)

    # Header for orders
    header_label = ctk.CTkLabel(
        orders_frame,
        text="Your Order History",
        font=("Helvetica", 18, "bold"),
        text_color=SHOPPING,
    )
    header_label.pack(anchor="w", pady=(0, 20))

    def load_order_history():
        # Clear existing items
        for widget in orders_frame.winfo_children()[1:]:  # Keep the header
            widget.destroy()

        try:
            headers = {"Authorization": f"Bearer {frame.token}"}
            print(
                f"Loading orders with token: {frame.token[:10] if frame.token else 'None'}"
            )
            response = requests.get(f"{API_URL}/order/list", headers=headers)
            print(f"Orders response status: {response.status_code}")

            if response.status_code == 200:
                orders = response.json()
                print(f"Orders received: {len(orders)}")

                if not orders:
                    empty_label = ctk.CTkLabel(
                        orders_frame,
                        text="You haven't placed any orders yet",
                        font=("Helvetica", 14),
                    )
                    empty_label.pack(pady=20)
                    return

                for order in orders:
                    order_frame = create_order_history_frame(order)
                    order_frame.pack(fill="x", pady=10)

            else:
                error_msg = response.json().get("detail", "Unknown error")
                print(f"Error loading orders: {error_msg}")
                messagebox.showerror("Error", f"Failed to load orders: {error_msg}")

        except Exception as e:
            print(f"Exception in load_order_history: {e}")
            messagebox.showerror("Error", f"Failed to load order history: {str(e)}")

    def create_order_history_frame(order):
        order_frame = ctk.CTkFrame(orders_frame, fg_color=CARD_BG, corner_radius=10)

        # Order header
        header_frame = ctk.CTkFrame(order_frame, fg_color="#212121", corner_radius=5)
        header_frame.pack(fill="x", padx=10, pady=(10, 0))

        # Order ID and date
        order_id = order.get("id", "Unknown")
        created_at = order.get("created_at", "Unknown date")
        if isinstance(created_at, str) and len(created_at) > 10:
            created_at = created_at[:10]  # Just get the date part

        header_text = f"Order #{order_id} - {created_at}"
        ctk.CTkLabel(
            header_frame, text=header_text, font=("Helvetica", 14, "bold")
        ).pack(side="left", padx=10, pady=5)

        # Order status
        status = order.get("status", "unknown").upper()
        status_colors = {
            "PENDING": "#FFA500",
            "PROCESSING": "#4169E1",
            "SHIPPED": "#9370DB",
            "DELIVERED": "#32CD32",
            "COMPLETED": "#32CD32",
            "CANCELLED": "#DC143C",
        }
        status_color = status_colors.get(status, "#808080")

        ctk.CTkLabel(
            header_frame,
            text=status,
            font=("Helvetica", 14, "bold"),
            text_color=status_color,
        ).pack(side="right", padx=10, pady=5)

        # Order details
        details_frame = ctk.CTkFrame(order_frame, fg_color="transparent")
        details_frame.pack(fill="x", padx=10, pady=10)

        # Delivery address
        address_frame = ctk.CTkFrame(details_frame, fg_color="transparent")
        address_frame.pack(fill="x", pady=5)

        ctk.CTkLabel(
            address_frame,
            text="Delivery Address:",
            font=("Helvetica", 12, "bold"),
            text_color="#AAAAAA",
        ).pack(side="left", padx=(10, 5))

        ctk.CTkLabel(
            address_frame,
            text=order.get("delivery_address", "No address provided"),
            font=("Helvetica", 12),
            text_color="white",
        ).pack(side="left")

        # Order items
        items_frame = ctk.CTkFrame(details_frame, fg_color="transparent")
        items_frame.pack(fill="x", pady=5)

        ctk.CTkLabel(
            items_frame,
            text="Ordered Items:",
            font=("Helvetica", 12, "bold"),
            text_color="#AAAAAA",
        ).pack(anchor="w", padx=(10, 5), pady=(5, 0))

        # Create a frame for each order item
        order_items = order.get("order_items", [])
        for item in order_items:
            item_frame = ctk.CTkFrame(items_frame, fg_color="#1e1e1e", corner_radius=5)
            item_frame.pack(fill="x", padx=10, pady=2)

            # Get product details
            try:
                headers = {"Authorization": f"Bearer {frame.token}"}
                product_id = item.get("product_id")
                response = requests.get(
                    f"{API_URL}/product/get/{product_id}", headers=headers
                )

                if response.status_code == 200:
                    product = response.json()
                    product_name = product.get("name", "Unknown Product")
                else:
                    product_name = f"Product #{product_id}"
            except:
                product_name = f"Product #{product_id}"

            # Product info
            product_info = ctk.CTkLabel(
                item_frame,
                text=f"{product_name} × {item.get('quantity', 0)}",
                font=("Helvetica", 12),
                text_color="white",
            )
            product_info.pack(side="left", padx=10, pady=5)

            # Item price
            item_price = item.get("price", 0) * item.get("quantity", 0)
            price_label = ctk.CTkLabel(
                item_frame,
                text=f"${item_price:.2f}",
                font=("Helvetica", 12),
                text_color="#AAAAAA",
            )
            price_label.pack(side="right", padx=10, pady=5)

        # Totals
        totals_frame = ctk.CTkFrame(details_frame, fg_color="transparent")
        totals_frame.pack(fill="x", pady=5)

        # Shipping price
        shipping_frame = ctk.CTkFrame(totals_frame, fg_color="transparent")
        shipping_frame.pack(fill="x")

        ctk.CTkLabel(
            shipping_frame,
            text="Shipping:",
            font=("Helvetica", 12, "bold"),
            text_color="#AAAAAA",
        ).pack(side="left", padx=(10, 5))

        shipping_price = order.get("shipping_price", 0)
        ctk.CTkLabel(
            shipping_frame,
            text=f"${shipping_price:.2f}",
            font=("Helvetica", 12),
            text_color="white",
        ).pack(side="left")

        # Total price
        total_frame = ctk.CTkFrame(totals_frame, fg_color="transparent")
        total_frame.pack(fill="x")

        ctk.CTkLabel(
            total_frame,
            text="Total:",
            font=("Helvetica", 14, "bold"),
            text_color="#AAAAAA",
        ).pack(side="left", padx=(10, 5))

        total_price = order.get("total_price", 0)
        ctk.CTkLabel(
            total_frame,
            text=f"${total_price:.2f}",
            font=("Helvetica", 14, "bold"),
            text_color=SHOPPING,
        ).pack(side="left")

        return order_frame

    # SHIPPED ORDERS TAB
    shipped_orders_frame = ctk.CTkScrollableFrame(shipped_tab, fg_color="transparent")
    shipped_orders_frame.pack(fill="both", expand=True, padx=20, pady=20)

    # Header for shipped orders
    shipped_header_label = ctk.CTkLabel(
        shipped_orders_frame,
        text="Shipped Orders",
        font=("Helvetica", 18, "bold"),
        text_color=SHOPPING,
    )
    shipped_header_label.pack(anchor="w", pady=(0, 20))

    def load_shipped_orders():
        # Clear existing items
        for widget in shipped_orders_frame.winfo_children()[1:]:  # Keep the header
            widget.destroy()

        try:
            headers = {"Authorization": f"Bearer {frame.token}"}
            response = requests.get(
                f"{API_URL}/order/list?status=shipped", headers=headers
            )

            if response.status_code == 200:
                orders = response.json()

                if not orders:
                    empty_label = ctk.CTkLabel(
                        shipped_orders_frame,
                        text="You don't have any shipped orders yet",
                        font=("Helvetica", 14),
                    )
                    empty_label.pack(pady=20)
                    return

                for order in orders:
                    order_frame = create_shipped_order_frame(order)
                    order_frame.pack(fill="x", pady=10)

            else:
                error_msg = response.json().get("detail", "Unknown error")
                print(f"Error loading shipped orders: {error_msg}")
                messagebox.showerror(
                    "Error", f"Failed to load shipped orders: {error_msg}"
                )

        except Exception as e:
            print(f"Exception in load_shipped_orders: {e}")
            messagebox.showerror("Error", f"Failed to load shipped orders: {str(e)}")

    def create_shipped_order_frame(order):
        order_frame = ctk.CTkFrame(
            shipped_orders_frame, fg_color=CARD_BG, corner_radius=10
        )

        # Order header
        header_frame = ctk.CTkFrame(
            order_frame, fg_color="#152e1d", corner_radius=5
        )  # Green tint for shipped
        header_frame.pack(fill="x", padx=10, pady=(10, 0))

        # Order ID and date
        order_id = order.get("id", "Unknown")
        created_at = order.get("created_at", "Unknown date")
        if isinstance(created_at, str) and len(created_at) > 10:
            created_at = created_at[:10]  # Just get the date part

        header_text = f"Order #{order_id} - {created_at}"
        ctk.CTkLabel(
            header_frame, text=header_text, font=("Helvetica", 14, "bold")
        ).pack(side="left", padx=10, pady=5)

        # Order status with checkmark
        ctk.CTkLabel(
            header_frame,
            text="✓ SHIPPED",
            font=("Helvetica", 14, "bold"),
            text_color="#32CD32",  # Bright green
        ).pack(side="right", padx=10, pady=5)

        # Order details
        details_frame = ctk.CTkFrame(order_frame, fg_color="transparent")
        details_frame.pack(fill="x", padx=10, pady=10)

        # Delivery address
        address_frame = ctk.CTkFrame(details_frame, fg_color="transparent")
        address_frame.pack(fill="x", pady=5)

        ctk.CTkLabel(
            address_frame,
            text="Delivery Address:",
            font=("Helvetica", 12, "bold"),
            text_color="#AAAAAA",
        ).pack(side="left", padx=(10, 5))

        ctk.CTkLabel(
            address_frame,
            text=order.get("delivery_address", "No address provided"),
            font=("Helvetica", 12),
            text_color="white",
        ).pack(side="left")

        # Expected delivery date (estimated)
        shipping_date = order.get("updated_at", "Unknown date")
        if isinstance(shipping_date, str) and len(shipping_date) > 10:
            shipping_date = shipping_date[:10]  # Just get the date part

        estimated_frame = ctk.CTkFrame(details_frame, fg_color="transparent")
        estimated_frame.pack(fill="x", pady=5)

        ctk.CTkLabel(
            estimated_frame,
            text="Shipped Date:",
            font=("Helvetica", 12, "bold"),
            text_color="#AAAAAA",
        ).pack(side="left", padx=(10, 5))

        ctk.CTkLabel(
            estimated_frame,
            text=shipping_date,
            font=("Helvetica", 12),
            text_color="white",
        ).pack(side="left")

        # Order items
        items_frame = ctk.CTkFrame(details_frame, fg_color="transparent")
        items_frame.pack(fill="x", pady=5)

        ctk.CTkLabel(
            items_frame,
            text="Shipped Items:",
            font=("Helvetica", 12, "bold"),
            text_color="#AAAAAA",
        ).pack(anchor="w", padx=(10, 5), pady=(5, 0))

        # Create a frame for each order item
        order_items = order.get("order_items", [])
        for item in order_items:
            item_frame = ctk.CTkFrame(items_frame, fg_color="#1e1e1e", corner_radius=5)
            item_frame.pack(fill="x", padx=10, pady=2)

            # Get product details
            try:
                headers = {"Authorization": f"Bearer {frame.token}"}
                product_id = item.get("product_id")
                response = requests.get(
                    f"{API_URL}/product/get/{product_id}", headers=headers
                )

                if response.status_code == 200:
                    product = response.json()
                    product_name = product.get("name", "Unknown Product")
                else:
                    product_name = f"Product #{product_id}"
            except:
                product_name = f"Product #{product_id}"

            # Product info
            product_info = ctk.CTkLabel(
                item_frame,
                text=f"{product_name} × {item.get('quantity', 0)}",
                font=("Helvetica", 12),
                text_color="white",
            )
            product_info.pack(side="left", padx=10, pady=5)

            # Item price
            item_price = item.get("price", 0) * item.get("quantity", 0)
            price_label = ctk.CTkLabel(
                item_frame,
                text=f"${item_price:.2f}",
                font=("Helvetica", 12),
                text_color="#AAAAAA",
            )
            price_label.pack(side="right", padx=10, pady=5)

        # Totals
        totals_frame = ctk.CTkFrame(details_frame, fg_color="transparent")
        totals_frame.pack(fill="x", pady=5)

        # Shipping price
        shipping_frame = ctk.CTkFrame(totals_frame, fg_color="transparent")
        shipping_frame.pack(fill="x")

        ctk.CTkLabel(
            shipping_frame,
            text="Shipping:",
            font=("Helvetica", 12, "bold"),
            text_color="#AAAAAA",
        ).pack(side="left", padx=(10, 5))

        shipping_price = order.get("shipping_price", 0)
        ctk.CTkLabel(
            shipping_frame,
            text=f"${shipping_price:.2f}",
            font=("Helvetica", 12),
            text_color="white",
        ).pack(side="left")

        # Total price
        total_frame = ctk.CTkFrame(totals_frame, fg_color="transparent")
        total_frame.pack(fill="x")

        ctk.CTkLabel(
            total_frame,
            text="Total:",
            font=("Helvetica", 14, "bold"),
            text_color="#AAAAAA",
        ).pack(side="left", padx=(10, 5))

        total_price = order.get("total_price", 0)
        ctk.CTkLabel(
            total_frame,
            text=f"${total_price:.2f}",
            font=("Helvetica", 14, "bold"),
            text_color=SHOPPING,
        ).pack(side="left")

        # Track package button
        button_frame = ctk.CTkFrame(details_frame, fg_color="transparent")
        button_frame.pack(fill="x", pady=(10, 5))

        def mark_as_received():
            if messagebox.askyesno(
                "Confirm Receipt", "Confirm that you have received this order?"
            ):
                try:
                    headers = {"Authorization": f"Bearer {frame.token}"}
                    response = requests.put(
                        f"{API_URL}/order/update_status/{order_id}",
                        json={"status": "delivered"},
                        headers=headers,
                    )

                    if response.status_code == 200:
                        messagebox.showinfo("Success", "Order marked as delivered!")
                        # Refresh shipped orders
                        load_shipped_orders()
                    else:
                        error_msg = response.json().get("detail", "Unknown error")
                        messagebox.showerror(
                            "Error", f"Failed to update order: {error_msg}"
                        )
                except Exception as e:
                    messagebox.showerror("Error", f"Failed to update order: {str(e)}")

        received_btn = ctk.CTkButton(
            button_frame,
            text="Mark as Received",
            command=mark_as_received,
            fg_color="#32CD32",  # Green color
            hover_color="#228B22",  # Darker green
            height=30,
        )
        received_btn.pack(side="right", padx=10)

        return order_frame

    # Load orders and cart when the frame is created
    load_order_history()
    load_cart_items()
    load_shipped_orders()

    # Check user role and update UI accordingly
    def check_user_role():
        nonlocal is_owner, owner_button
        headers = {"Authorization": f"Bearer {frame.token}"}
        try:
            resp = requests.get(f"{API_URL}/user/profile", headers=headers)
            if resp.status_code == 200:
                data = resp.json()

                # Check if user is an owner and update UI accordingly
                user_role = data.get("role", "")
                is_owner = user_role == "owner" or user_role == "shop_owner"

                # Remove any existing button in the owner button frame
                for widget in owner_button_frame.winfo_children():
                    widget.destroy()

                # Create the appropriate button based on user role
                if is_owner:
                    # Create a button with the same styling as other nav buttons
                    owner_button = ctk.CTkButton(
                        owner_button_frame,
                        text="Go to Shop Dashboard",
                        fg_color=SHOPPING,  # Use the shopping color for emphasis
                        text_color="white",
                        hover_color="#0096ff",
                        height=40,
                        command=go_to_owner_dashboard,
                        font=("Helvetica", 14, "bold"),
                    )
                else:
                    # Create a styled button that stands out better than the nav buttons
                    owner_button = ctk.CTkButton(
                        owner_button_frame,
                        text="Become Shop Owner",
                        command=open_create_shop,
                        fg_color="#3b3b3b",  # Darker than transparent but not too prominent
                        text_color="white",
                        hover_color="#4a4a4a",
                        height=40,
                        font=("Helvetica", 14),
                    )

                # Pack without any padding to match other buttons
                owner_button.pack(fill="x")
        except Exception as e:
            print(f"Error checking user role: {e}")

    # Call role check on initialization
    check_user_role()

    return frame
